package host.plugin;

import host.extensionPoints.Calculation;
import host.extensionPoints.Speech;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Set;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import plugin.AbstractPlugin;
import plugin.Extension;
import plugin.ExtensionPoint;


public class MainPlugin extends AbstractPlugin implements ActionListener{
    JTextArea output;
    JScrollPane scrollPane;
    private static String key="clarkson.plugin.extension";

    public MainPlugin(){
    	super("clarkson.base");
    }
    
    public void actionPerformed(ActionEvent e){
    	JMenuItem menuItem = (JMenuItem)e.getSource();
    	Extension ext = (Extension)menuItem.getClientProperty(key);
    	String clazzName = ext.getClazz();
    	String id = ext.getId();
		System.out.println(id+" "+clazzName+loader);
		try {
    	Class clazz = ext.plugin.loader.loadClass(clazzName);
    	if ("clarkson.calculator".equals(id)){
    		Calculation calc = (Calculation)clazz.newInstance();
    		calc.calc(output);
    		}
    	if ("clarkson.speech".equals(id)){
    		Speech speech = (Speech)clazz.newInstance();
    		speech.speak(output);
    		}
    	}
		catch (Exception ex){
			System.err.println(ex);
		}
    }
    public JMenuBar createMenuBar() {
        JMenuBar menuBar;
        JMenu menu;
        JMenuItem menuItem;

        //Create the menu bar.
        menuBar = new JMenuBar();

        Set eps = getExtensionPoints();
    	System.out.println("In MainPlugin "+eps.size());
    	for (Iterator it=eps.iterator(); it.hasNext();){
        	ExtensionPoint ep = (ExtensionPoint)it.next();

            //Build a menu.
            menu = new JMenu(ep.getName());
//            menu.setMnemonic(KeyEvent.VK_A);
//            menu.getAccessibleContext().setAccessibleDescription(
//                    "From here you can choose from many calculators");
            menuBar.add(menu);
            ArrayList exts = ep.getExtensions();
            for (int i=0; i<exts.size(); ++i){
            	Extension ext = (Extension)exts.get(i);
            	menuItem = new JMenuItem(ext.getName());
            	menuItem.putClientProperty(key, ext);
            	menuItem.addActionListener(this);
                menu.add(menuItem);            	
            }
      }
 
        return menuBar;
    }

    public Container createContentPane() {
        //Create the content-pane-to-be.
        JPanel contentPane = new JPanel(new BorderLayout());
        contentPane.setOpaque(true);

        //Create a scrolled text area.
        output = new JTextArea(5, 30);
        output.setEditable(false);
        scrollPane = new JScrollPane(output);

        //Add the text area to the content pane.
        contentPane.add(scrollPane, BorderLayout.CENTER);

        return contentPane;
    }

    /** Returns an ImageIcon, or null if the path was invalid. */
    protected static ImageIcon createImageIcon(String path) {
        java.net.URL imgURL = MainPlugin.class.getResource(path);
        if (imgURL != null) {
            return new ImageIcon(imgURL);
        } else {
            System.err.println("Couldn't find file: " + path);
            return null;
        }
    }

    /**
     * Create the GUI and show it.  For thread safety,
     * this method should be invoked from the
     * event-dispatching thread.
     */
    private void createAndShowGUI() {
        //Create and set up the window.
        JFrame frame = new JFrame("Clarskon Plugin Demo");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        //Create and set up the content pane.
        frame.setJMenuBar(createMenuBar());
        frame.setContentPane(createContentPane());

        //Display the window.
        frame.setSize(450, 260);
        frame.setVisible(true);
    }

    public void run(String[] args) {
        //Schedule a job for the event-dispatching thread:
        //creating and showing this application's GUI.
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                createAndShowGUI();
            }
        });
    }
}
