package plugin;

import java.io.File;
import java.io.InputStream;
import java.util.jar.JarFile;
import java.util.zip.ZipEntry;

public class PluginClassLoader extends ClassLoader {

	public AbstractPlugin plugin;
	
	private String jarPath;

	/**
	 * delegation constructor
	 */
	public PluginClassLoader(ClassLoader parent, String jar) {
		super(parent);
		this.jarPath = jar;
	}

	/**
	 * Same old ClassLoader constructor
	 * 
	 */
	public PluginClassLoader(String jar) {
		super();
		this.jarPath = jar;
	}

	/**
	 * Load a class from the local store identified by root.
	 * 
	 * @param className  
	 *               the fully-qualified name of the class
	 * @return the resulting <code>Class</code> object
	 * @exception ClassNotFoundException
	 *               if the class could not be found
	 */
	protected Class findClass(String className) throws ClassNotFoundException 
	{
		String msg = "Class "+className+" is found and loaded.";
		InputStream is = null;

		try 
		{
			System.out.println("LocalClassLoader is searching for class: " + className);
			String entryName = className.replace('.', '/')+".class";			
			JarFile jar = new JarFile(new File(jarPath));
//			Enumeration enumeration = jar.entries();
//			while (enumeration.hasMoreElements()){
//				System.out.println(enumeration.nextElement());				
//			}
//			System.out.println(jar);
			ZipEntry ze = jar.getEntry(entryName);
			System.out.println(ze);
			is = jar.getInputStream(ze);
			System.out.println(is);
			byte[] classBytes = new byte[is.available()];
			is.read(classBytes);
			
			definePackage(className);
			return defineClass(className, classBytes, 0, classBytes.length);
		} 
		catch (Exception e) 
		{
			// We cannot find the class, so indicate the problem with an exception
			msg = "LocalClassLoader cannot find class: ";
			throw new ClassNotFoundException(className);
		} 
		finally 
		{
			System.out.println(msg);
			if (null != is) 
			{
				try 
				{
					is.close();
				} 
				catch (Exception e) 
				{
				}
			}
		}
	}

	/**
	 * Minimal package definition
	 * 
	 */
	private void definePackage(String className) {
		// Extract the package name from the class name,
		String pkgName = className;
		int index = className.lastIndexOf('.');
		if (-1 != index) {
			pkgName = className.substring(0, index);
		}

		if (getPackage(pkgName) != null) 
		{
			return;
		}

		definePackage(pkgName, "JLS", "2.0", "sun", "sun", "1.4.2", "sun", null); // no sealing for simplicity
	}

}
